module PowerModelsADA

import JuMP
import PowerModels
import InfrastructureModels
import Serialization
import LinearAlgebra
import DelimitedFiles
import SparseArrays
import Suppressor: @capture_out
import Distributed

import PowerModels: AbstractPowerModel, parse_file, ids, ref, var, con, sol, nw_ids, nws, optimize_model!, update_data!, ref_add_core!, pm_it_sym, pm_it_name, nw_id_default, ismultinetwork, ismulticonductor, silence

const _PM = PowerModels
const _IM = InfrastructureModels
# const _pmada_global_keys = Set(["time_series", "per_unit", "parameter", "option", "solution", "local_solution", "shared_variable", "received_variable", "received_delta", "dual_variable", "received_dual_variable", "shared_sensitivities", "shared_dual_variable", "dual_residual", "mismatch", "dual_residual", "counter"])

const _pmada_global_keys = Set(["time_series", "per_unit", "parameter", "option", "solution", "mismatch", "counter", "previous_solution", "shared_flag_convergence", "received_flag_convergence", "shared_convergence_iteration", "received_convergence_iteration"])

include("core/base.jl")
include("core/variables.jl")
include("core/opf.jl")
include("core/data.jl")
include("core/data_sharing.jl")
include("core/util.jl")
include("core/export.jl")

include("algorithms/admm_methods.jl")
include("algorithms/atc_methods.jl")
include("algorithms/app_methods.jl")
include("algorithms/admm_coordinated_methods.jl")
include("algorithms/atc_coordinated_methods.jl")
include("algorithms/aladin_coordinated_methods.jl")
include("algorithms/adaptive_admm_methods.jl")
include("algorithms/adaptive_admm_coordinated_methods.jl")



end